@@ -94,6 +94,8 @@ parseconf_bool_array[] =
   { "ssl_sslv3", &tunable_sslv3 },
   { "ssl_tlsv1", &tunable_tlsv1 },
   { "tilde_user_enable", &tunable_tilde_user_enable },
+  { "force_anon_logins_ssl", &tunable_force_anon_logins_ssl },
+  { "force_anon_data_ssl", &tunable_force_anon_data_ssl },
   { 0, 0 }
 };
 
@@ -158,6 +160,8 @@ parseconf_str_array[] =
   { "rsa_cert_file", &tunable_rsa_cert_file },
   { "dsa_cert_file", &tunable_dsa_cert_file },
   { "ssl_ciphers", &tunable_ssl_ciphers },
+  { "rsa_private_key_file", &tunable_rsa_private_key_file },
+  { "dsa_private_key_file", &tunable_dsa_private_key_file },
   { 0, 0 }
 };
 
