@@ -54,9 +54,8 @@ emit_greeting(struct vsf_session* p_sess)
   {
     str_alloc_text(&str_log_line, "Connection refused: too many sessions.");
     vsf_log_line(p_sess, kVSFLogEntryConnection, &str_log_line);
-    vsf_cmdio_write_noblock(p_sess, FTP_TOO_MANY_USERS,
-                    "There are too many connected users, please try later.");
-    vsf_sysutil_exit(0);
+    vsf_cmdio_write_exit(p_sess, FTP_TOO_MANY_USERS,
+      "There are too many connected users, please try later.");
   }
   if (tunable_max_per_ip > 0 &&
       p_sess->num_this_ip > tunable_max_per_ip)
@@ -64,17 +63,15 @@ emit_greeting(struct vsf_session* p_sess)
     str_alloc_text(&str_log_line,
                    "Connection refused: too many sessions for this address.");
     vsf_log_line(p_sess, kVSFLogEntryConnection, &str_log_line);
-    vsf_cmdio_write_noblock(p_sess, FTP_IP_LIMIT,
-        "There are too many connections from your internet address.");
-    vsf_sysutil_exit(0);
+    vsf_cmdio_write_exit(p_sess, FTP_IP_LIMIT,
+      "There are too many connections from your internet address.");
   }
   if (!p_sess->tcp_wrapper_ok)
   {
     str_alloc_text(&str_log_line,
                    "Connection refused: tcp_wrappers denial.");
     vsf_log_line(p_sess, kVSFLogEntryConnection, &str_log_line);
-    vsf_cmdio_write_noblock(p_sess, FTP_IP_DENY, "Service not available.");
-    vsf_sysutil_exit(0);
+    vsf_cmdio_write_exit(p_sess, FTP_IP_DENY, "Service not available.");
   }
   vsf_log_line(p_sess, kVSFLogEntryConnection, &str_log_line);
   if (!str_isempty(&p_sess->banner_str))
@@ -160,7 +157,8 @@ handle_user_command(struct vsf_session* p_sess)
     str_empty(&p_sess->user_str);
     return;
   }
-  if (is_anon && p_sess->control_use_ssl && !tunable_allow_anon_ssl)
+  if (is_anon && p_sess->control_use_ssl && !tunable_allow_anon_ssl &&
+      !tunable_force_anon_logins_ssl)
   {
     vsf_cmdio_write(
       p_sess, FTP_LOGINERR, "Anonymous sessions may not use encryption.");
@@ -175,6 +173,14 @@ handle_user_command(struct vsf_session* p_sess)
     str_empty(&p_sess->user_str);
     return;
   }
+  if (tunable_ssl_enable && is_anon && !p_sess->control_use_ssl &&
+      tunable_force_anon_logins_ssl)
+  { 
+    vsf_cmdio_write(
+      p_sess, FTP_LOGINERR, "Anonymous sessions must use encryption.");
+    str_empty(&p_sess->user_str);
+    return;
+  }
   if (!str_isempty(&p_sess->userlist_str))
   {
     int located = str_contains_line(&p_sess->userlist_str, &p_sess->user_str);
