@@ -21,18 +21,19 @@
 #include "utility.h"
 #include "builddefs.h"
 
-#ifdef VSF_BUILD_SSL
-  #include <openssl/ssl.h>
-#endif
-
 #ifdef VSF_BUILD_SSL
 
+#include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/rand.h>
+#include <openssl/bio.h>
 
 static char* get_ssl_error();
 static SSL* get_ssl(struct vsf_session* p_sess, int fd);
 static int ssl_session_init(struct vsf_session* p_sess);
+static void setup_bio_callbacks();
+static long bio_callback(
+  BIO* p_bio, int oper, const char* p_arg, int argi, long argl, long retval);
 
 static int ssl_inited;
 
@@ -65,31 +66,40 @@ ssl_init(struct vsf_session* p_sess)
     SSL_CTX_set_options(p_ctx, options);
     if (tunable_rsa_cert_file)
     {
+      const char* p_key = tunable_rsa_private_key_file;
+      if (!p_key)
+      {
+        p_key = tunable_rsa_cert_file;
+      }
       if (SSL_CTX_use_certificate_file(
         p_ctx, tunable_rsa_cert_file, X509_FILETYPE_PEM) != 1)
       {
-        die("SSL: cannot load RSA key");
+        die("SSL: cannot load RSA certificate");
       }
-      if (SSL_CTX_use_PrivateKey_file(
-        p_ctx, tunable_rsa_cert_file, X509_FILETYPE_PEM) != 1)
+      if (SSL_CTX_use_PrivateKey_file(p_ctx, p_key, X509_FILETYPE_PEM) != 1)
       {
-        die("SSL: cannot load RSA key");
+        die("SSL: cannot load RSA private key");
       }
     }
     if (tunable_dsa_cert_file)
     {
+      const char* p_key = tunable_dsa_private_key_file;
+      if (!p_key)
+      {
+        p_key = tunable_dsa_cert_file;
+      }
       if (SSL_CTX_use_certificate_file(
         p_ctx, tunable_dsa_cert_file, X509_FILETYPE_PEM) != 1)
       {
-        die("SSL: cannot load DSA key");
+        die("SSL: cannot load DSA certificate");
       }
-      if (SSL_CTX_use_PrivateKey_file(
-        p_ctx, tunable_dsa_cert_file, X509_FILETYPE_PEM) != 1)
+      if (SSL_CTX_use_PrivateKey_file(p_ctx, p_key, X509_FILETYPE_PEM) != 1)
       {
-        die("SSL: cannot load DSA key");
+        die("SSL: cannot load DSA private key");
       }
     }
-    if (SSL_CTX_set_cipher_list(p_ctx, tunable_ssl_ciphers) != 1)
+    if (tunable_ssl_ciphers &&
+        SSL_CTX_set_cipher_list(p_ctx, tunable_ssl_ciphers) != 1)
     {
       die("SSL: could not set cipher list");
     }
@@ -203,18 +213,30 @@ ssl_getline(const struct vsf_session* p_sess, struct mystr* p_str,
 int
 ssl_read(void* p_ssl, char* p_buf, unsigned int len)
 {
-  int retval = SSL_read((SSL*) p_ssl, p_buf, len);
-  int fd = SSL_get_fd((SSL*) p_ssl);
-  vsf_sysutil_check_pending_actions(kVSFSysUtilIO, retval, fd);
+  int retval;
+  int err;
+  do
+  {
+    retval = SSL_read((SSL*) p_ssl, p_buf, len);
+    err = SSL_get_error((SSL*) p_ssl, retval);
+  }
+  while (retval < 0 && (err == SSL_ERROR_WANT_READ ||
+                        err == SSL_ERROR_WANT_WRITE));
   return retval;
 }
 
 int
 ssl_write(void* p_ssl, const char* p_buf, unsigned int len)
 {
-  int retval = SSL_write((SSL*) p_ssl, p_buf, len);
-  int fd = SSL_get_fd((SSL*) p_ssl);
-  vsf_sysutil_check_pending_actions(kVSFSysUtilIO, retval, fd);
+  int retval;
+  int err;
+  do
+  {
+    retval = SSL_write((SSL*) p_ssl, p_buf, len);
+    err = SSL_get_error((SSL*) p_ssl, retval);
+  }
+  while (retval < 0 && (err == SSL_ERROR_WANT_READ ||
+                        err == SSL_ERROR_WANT_WRITE));
   return retval;
 }
 
@@ -239,6 +261,7 @@ ssl_accept(struct vsf_session* p_sess, int fd)
     return 0;
   }
   p_sess->p_data_ssl = p_ssl;
+  setup_bio_callbacks(p_ssl);
   return 1;
 }
 
@@ -288,6 +311,7 @@ ssl_session_init(struct vsf_session* p_sess)
     return 0;
   }
   p_sess->p_control_ssl = p_ssl;
+  setup_bio_callbacks(p_ssl);
   return 1;
 }
 
@@ -298,6 +322,33 @@ get_ssl_error()
   return ERR_error_string(ERR_get_error(), NULL);
 }
 
+static void setup_bio_callbacks(SSL* p_ssl)
+{
+  BIO* p_bio = SSL_get_rbio(p_ssl);
+  BIO_set_callback(p_bio, bio_callback);
+  p_bio = SSL_get_wbio(p_ssl);
+  BIO_set_callback(p_bio, bio_callback);
+}
+
+static long
+bio_callback(
+  BIO* p_bio, int oper, const char* p_arg, int argi, long argl, long ret)
+{
+  int retval = 0;
+  int fd = 0;
+  (void) p_arg;
+  (void) argi;
+  (void) argl;
+  if (oper == (BIO_CB_READ | BIO_CB_RETURN) ||
+      oper == (BIO_CB_WRITE | BIO_CB_RETURN))
+  {
+    retval = (int) ret;
+    fd = BIO_get_fd(p_bio, NULL);
+  }
+  vsf_sysutil_check_pending_actions(kVSFSysUtilIO, retval, fd);
+  return ret;
+}
+
 #else /* VSF_BUILD_SSL */
 
 void
