@@ -620,7 +620,7 @@ vsf_sysutil_activate_keepalive(int fd)
                           sizeof(keepalive));
   if (retval != 0)
   {
-    die("setsockopt");
+    die("setsockopt: keepalive");
   }
 }
 
@@ -632,7 +632,7 @@ vsf_sysutil_activate_reuseaddr(int fd)
                           sizeof(reuseaddr));
   if (retval != 0)
   {
-    die("setsockopt");
+    die("setsockopt: reuseaddr");
   }
 }
 
@@ -644,7 +644,7 @@ vsf_sysutil_set_nodelay(int fd)
                           sizeof(nodelay));
   if (retval != 0)
   {
-    die("setsockopt");
+    die("setsockopt: nodelay");
   }
 }
 
@@ -666,7 +666,7 @@ vsf_sysutil_activate_oobinline(int fd)
                           sizeof(oob_inline));
   if (retval != 0)
   {
-    die("setsockopt");
+    die("setsockopt: oobinline");
   }
 }
 
@@ -683,13 +683,14 @@ vsf_sysutil_activate_linger(int fd)
 {
   int retval;
   struct linger the_linger;
+  vsf_sysutil_memclr(&the_linger, sizeof(the_linger));
   the_linger.l_onoff = 1;
-  the_linger.l_linger = INT_MAX;
+  the_linger.l_linger = 32767;
   retval = setsockopt(fd, SOL_SOCKET, SO_LINGER, &the_linger,
                       sizeof(the_linger));
   if (retval != 0)
   {
-    die("setsockopt");
+    die("setsockopt: linger");
   }
 }
 
@@ -1789,6 +1790,16 @@ vsf_sysutil_shutdown_failok(int fd)
   (void) shutdown(fd, SHUT_RDWR);
 }
 
+void
+vsf_sysutil_shutdown_read_failok(int fd)
+{
+  /* SHUT_RD is a relatively new addition */
+  #ifndef SHUT_RD
+  #define SHUT_RD 0
+  #endif
+  (void) shutdown(fd, SHUT_RD);
+}
+
 void
 vsf_sysutil_sockaddr_clear(struct vsf_sysutil_sockaddr** p_sockptr)
 {
@@ -1909,17 +1920,6 @@ vsf_sysutil_sockaddr_is_ipv6(const struct vsf_sysutil_sockaddr* p_sockaddr)
   return 0;
 }
 
-int
-vsf_sysutil_sockaddr_same_family(const struct vsf_sysutil_sockaddr* p1,
-                                 const struct vsf_sysutil_sockaddr* p2)
-{
-  if (p1->u.u_sockaddr.sa_family == p2->u.u_sockaddr.sa_family)
-  {
-    return 1;
-  }
-  return 0;
-}
-
 void
 vsf_sysutil_sockaddr_set_ipv4addr(struct vsf_sysutil_sockaddr* p_sockptr,
                                   const unsigned char* p_raw)
@@ -1929,6 +1929,16 @@ vsf_sysutil_sockaddr_set_ipv4addr(struct vsf_sysutil_sockaddr* p_sockptr,
     vsf_sysutil_memcpy(&p_sockptr->u.u_sockaddr_in.sin_addr, p_raw,
                        sizeof(p_sockptr->u.u_sockaddr_in.sin_addr));
   }
+  else if (p_sockptr->u.u_sockaddr.sa_family == AF_INET6)
+  {
+    static struct vsf_sysutil_sockaddr* s_p_sockaddr;
+    vsf_sysutil_sockaddr_alloc_ipv4(&s_p_sockaddr);
+    vsf_sysutil_memcpy(&s_p_sockaddr->u.u_sockaddr_in.sin_addr, p_raw,
+                       sizeof(&s_p_sockaddr->u.u_sockaddr_in.sin_addr));
+    vsf_sysutil_memcpy(&p_sockptr->u.u_sockaddr_in6.sin6_addr,
+                       vsf_sysutil_sockaddr_ipv4_v6(s_p_sockaddr),
+                       sizeof(p_sockptr->u.u_sockaddr_in6.sin6_addr));
+  }
   else
   {
     bug("bad family");
@@ -1967,6 +1977,18 @@ vsf_sysutil_sockaddr_ipv6_v4(const struct vsf_sysutil_sockaddr* p_addr)
   return &p_addr_start[12];
 }
 
+const void*
+vsf_sysutil_sockaddr_ipv4_v6(const struct vsf_sysutil_sockaddr* p_addr)
+{
+  static char ret[16] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0xFF, 0xFF };
+  if (p_addr->u.u_sockaddr.sa_family != AF_INET)
+  {
+    return 0;
+  }
+  vsf_sysutil_memcpy(&ret[12], &p_addr->u.u_sockaddr_in.sin_addr, 4);
+  return ret;
+}
+
 void*
 vsf_sysutil_sockaddr_get_raw_addr(struct vsf_sysutil_sockaddr* p_sockptr)
 {
