@@ -66,6 +66,8 @@ int tunable_sslv2 = 0;
 int tunable_sslv3 = 0;
 int tunable_tlsv1 = 1;
 int tunable_tilde_user_enable = 0;
+int tunable_force_anon_logins_ssl = 0;
+int tunable_force_anon_data_ssl = 0;
 
 unsigned int tunable_accept_timeout = 60;
 unsigned int tunable_connect_timeout = 60;
@@ -115,4 +117,6 @@ const char* tunable_email_password_file = "/etc/vsftpd.email_passwords";
 const char* tunable_rsa_cert_file = "/usr/share/ssl/certs/vsftpd.pem";
 const char* tunable_dsa_cert_file = 0;
 const char* tunable_ssl_ciphers = "DES-CBC3-SHA";
+const char* tunable_rsa_private_key_file = 0;
+const char* tunable_dsa_private_key_file = 0;
 
